package org.jeecg.modules.okki.translate.service;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.okki.translate.dto.RransResultBaidu;
import org.jeecg.modules.okki.translate.dto.TransRresultSigle;
import org.jeecg.modules.okki.translate.dto.TranslateRequestDto;
import org.jeecg.modules.okki.translate.dto.TranslateResults;
import org.jeecg.modules.okki.translate.tools.TransBaiduApi;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.util.*;
import java.util.concurrent.*;

@Slf4j
@Service
public class TranslateService {

    @Autowired
    private TransBaiduApi transBaiduApi;

    private final ExecutorService executorService = Executors.newFixedThreadPool(10);

    public TranslateResults doTranslate(TranslateRequestDto translateRequestDto) throws UnsupportedEncodingException {
        // TODO 站点信息校验

        String fromLanguage = translateRequestDto.getFrom();
        List<String> targetLanguages = translateRequestDto.getTo();
        Map<String, String> contents = translateRequestDto.getContents();

        TranslateResults results = new TranslateResults();
        Map<String, Map<String, String>> translatedInfo = new ConcurrentHashMap<>();

        List<Future<Void>> futures = new ArrayList<>();

        // 预解码所有内容
        Map<String, String> decodedContents = new HashMap<>();
        for (Map.Entry<String, String> entry : contents.entrySet()) {
            decodedContents.put(entry.getKey(), new String(Base64.getDecoder().decode(entry.getValue())));
        }

        for (String targetLanguage : targetLanguages) {
            for (Map.Entry<String, String> entry : decodedContents.entrySet()) {
                futures.add(executorService.submit(() -> {
                    processTranslation(entry.getKey(), entry.getValue(), fromLanguage, targetLanguage, translatedInfo, contents.get(entry.getKey()));
                    return null;
                }));
            }
        }

        // 等待所有任务完成
        for (Future<Void> future : futures) {
            try {
                future.get();
            } catch (InterruptedException | ExecutionException e) {
                log.error("翻译任务执行出错", e);
                Thread.currentThread().interrupt();  // 处理中断
            }
        }

        results.setMsg("successful");
        results.setData(translatedInfo);

        return results;
    }

    private void processTranslation(String key, String decodedSrc, String fromLanguage, String targetLanguage, Map<String, Map<String, String>> translatedInfo, String originalValue) {
        try {
            String transObj = transBaiduApi.getTransResult(decodedSrc, fromLanguage, targetLanguage);

            RransResultBaidu resultBaidu = JSON.parseObject(transObj, RransResultBaidu.class);

            if (resultBaidu.getErrorCode() == null && !resultBaidu.getTransResult().isEmpty()) {
                for (TransRresultSigle singleResult : resultBaidu.getTransResult()) {
                    String encodedDst = Base64.getEncoder().encodeToString(singleResult.getDst().getBytes());

                    Map<String, String> trans = translatedInfo.computeIfAbsent(key, k -> new HashMap<>());
                    trans.put(targetLanguage, encodedDst);
                    trans.put("_original", originalValue);
                }
            } else {
                log.error("调用百度翻译发生错误！error_code:{} error_msg:{}", resultBaidu.getErrorCode(), resultBaidu.getErrorMsg());
            }
        } catch (Exception e) {
            log.error("处理翻译时发生异常", e);
        }
    }
}
